package com.ruoyi.system.utils.jsch;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.system.kernel.ServerLogKernel;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

public class ServiceShell {
   private Session session = null;
   private Channel channel = null;
   private InputStream in = null;
   private OutputStream out = null;
   private static final int IDLE_TIMEOUT_MINUTES = 600;
   private static String COMMAND_PROMPT = "command_send_success";
   private Long serverId = null;
   private ServerLogKernel serverLogKernel = null;

   public ServiceShell() {
   }

   public ServiceShell(Long serverId, ServerLogKernel serverLogKernel) {
      this.serverId = serverId;
      this.serverLogKernel = serverLogKernel;
   }

   public void connect(String host, String username, String password, int port) {
      try {
         JSch jSch = new JSch();
         this.session = jSch.getSession(username, host, port);
         this.session.setPassword(password);
         this.session.setUserInfo(new MyUserInfo());
         this.session.connect(30000);
         this.channel = this.session.openChannel("shell");
         this.in = this.channel.getInputStream();
         this.out = this.channel.getOutputStream();
         this.channel.connect(3000);
         if (!Objects.equals(username, "root")) {
            this.sendCommand("sudo su -");
         } else {
            this.readCommandOutput("", (Integer)null);
         }

      } catch (Exception var6) {
         Exception e = var6;
         throw new RuntimeException("连接服务器失败", e);
      }
   }

   public String sendCommand(String command) {
      try {
         this.send(command + "\n");
         return this.readCommandOutput(command, (Integer)null);
      } catch (IOException var3) {
         IOException e = var3;
         throw new RuntimeException("Failed to send command", e);
      }
   }

   public String sendCommand(String command, Integer timeoutMinute) {
      try {
         this.send(command + "\n");
         return this.readCommandOutput(command, timeoutMinute);
      } catch (IOException var4) {
         IOException e = var4;
         throw new RuntimeException("Failed to send command", e);
      }
   }

   private String readCommandOutput(String command, Integer timeoutMinute) throws IOException {
      if (timeoutMinute == null || timeoutMinute == 0) {
         timeoutMinute = 2;
      }

      int timeout = timeoutMinute * 600;
      String outStr = "空闲弹出，" + timeoutMinute + "分钟";
      StringBuilder output = new StringBuilder();
      byte[] buffer = new byte[1024];
      int num = 0;
      boolean endOfCommand = false;

      while(!endOfCommand) {
         if (this.in.available() <= 0) {
            ++num;
         } else {
            num = 0;
         }

         if (num > timeout) {
            System.out.println(outStr);
            output.append(outStr);
            break;
         }

         while(this.in.available() > 0) {
            int bytesRead = this.in.read(buffer, 0, 1024);
            if (bytesRead < 0) {
               endOfCommand = true;
               break;
            }

            String part = new String(buffer, 0, bytesRead, StandardCharsets.UTF_8);
            if (StringUtils.isEmpty(part)) {
               ++num;
            } else {
               num = 0;
            }

            if (num > timeout) {
               System.out.println(outStr);
               output.append(outStr);
               endOfCommand = true;
               break;
            }

            if (!StringUtils.isEmpty(part)) {
               output.append(part);
               if (this.isEndOfCommand(part)) {
                  endOfCommand = true;
                  break;
               }

               if (command.startsWith("sudo certbot certonly")) {
                  this.handleCertbotPrompts(part);
               }
            }
         }

         if (endOfCommand) {
            break;
         }

         try {
            Thread.sleep(100L);
         } catch (InterruptedException var11) {
            Thread.currentThread().interrupt();
            throw new RuntimeException("Thread interrupted", var11);
         }
      }

      return output.toString();
   }

   private boolean isEndOfCommand(String part) {
      return part.contains("root@") && (part.contains(":~$") || part.contains(":~#") || part.contains("~]#") || part.contains(COMMAND_PROMPT));
   }

   private void handleCertbotPrompts(String part) throws IOException {
      if (part.contains("Select the appropriate number [1-2]")) {
         this.send("2\n");
      }

      if (part.contains("(Y)es/(N)o")) {
         this.send("N\n");
      }

   }

   private void send(String command) throws IOException {
      if (this.out != null) {
         this.out.write(command.getBytes());
         this.out.flush();
      }

   }

   public String executeCommand(String command) {
      StringBuilder output = new StringBuilder();
      Channel channel = null;

      String var5;
      try {
         channel = this.session.openChannel("exec");
         ((ChannelExec)channel).setCommand(command);
         channel.setInputStream((InputStream)null);
         channel.connect();
         InputStream in = channel.getInputStream();

         try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(in));

            String line;
            try {
               while((line = reader.readLine()) != null) {
                  output.append(line).append("\n");
               }
            } catch (Throwable var17) {
               try {
                  reader.close();
               } catch (Throwable var16) {
                  var17.addSuppressed(var16);
               }

               throw var17;
            }

            reader.close();
         } catch (Throwable var18) {
            if (in != null) {
               try {
                  in.close();
               } catch (Throwable var15) {
                  var18.addSuppressed(var15);
               }
            }

            throw var18;
         }

         if (in != null) {
            in.close();
         }

         return output.toString().trim();
      } catch (JSchException | IOException var19) {
         var5 = "";
      } finally {
         if (channel != null && channel.isConnected()) {
            channel.disconnect();
         }

      }

      return var5;
   }

   public void disconnect() {
      if (this.channel != null) {
         this.channel.disconnect();
      }

      if (this.session != null) {
         this.session.disconnect();
      }

   }

   public static void main(String[] args) {
      ServiceShell shell = new ServiceShell();

      try {
         shell.connect("43.153.3.62", "ubuntu", "931yELfCHEwT", 22);
         String logs = shell.executeCommand("sudo docker logs --tail 100 sync-mysql");
         System.out.println(logs);
         shell.disconnect();
      } catch (Exception var3) {
         Exception e = var3;
         System.out.println(e.getMessage());
         shell.disconnect();
      }

   }
}
